import fetch from 'isomorphic-fetch';
import prepareUrls from './prepare_urls';
import path from 'path';
import { processRegister } from './process_manager';
import validateConfig from './validate_config';
import getWebpackConfigs from './get_webpack_configs';
import getPeerDependencies from './get_peer_dependencies';
import webpack from 'webpack';
import fsExtra from 'fs-extra';
import showBuildInfo from './show_build_info';
import writePackageJsonsForAssemblies from './write_package_jsons_for_assemblies';
import copyEnvToDist from './copy_env_to_dist';
import { getLog } from './get-log';
import detectErrors from './detect_errors';
import adjustResolveConfig from './adjust-resolve-config';
import getEntryOptions from './get_entry_options';

const waitForUrl = async (log, host, port, rootPath, apiHandlerUrl) => {
  const urls = prepareUrls('http', host, port, rootPath);
  const baseUrl = urls.localUrlForBrowser;
  const url = `${baseUrl}api/${apiHandlerUrl}`;
  log.debug(`target API url to fetch ${url}`);

  while (true) {
    try {
      const response = await fetch(url);
      const result = await response.text();

      if (result !== 'ok') {
        throw [`Error communicating with reSolve HTTP server at port ${port}`, `Multiple instances of reSolve applications may be trying to run on the same port`, `${response.status}: ${response.statusText}`, `${result}`].join('\n');
      }

      break;
    } catch (e) {}

    await new Promise(resolve => setTimeout(resolve, 500));
  }
};

const generateCustomMode = (getConfig, apiHandlerUrl, runAfterLaunch) => (resolveConfig, options, adjustWebpackConfigs) => new Promise(async (resolve, reject) => {
  const apiHandlerUrls = [].concat(apiHandlerUrl);
  const log = getLog(`start:${apiHandlerUrls.join('+')}`);

  try {
    log.debug(`validating framework config`);
    await adjustResolveConfig(resolveConfig);
    const config = await getConfig(resolveConfig, options);
    validateConfig(config);
    log.debug(`requesting webpack configs`);
    const nodeModulesByAssembly = new Map();
    const webpackConfigs = await getWebpackConfigs({
      resolveConfig: config,
      nodeModulesByAssembly,
      adjustWebpackConfigs
    });
    const peerDependencies = getPeerDependencies();
    log.debug(`creating webpack compiler`);
    const compiler = webpack(webpackConfigs);
    log.debug(`injecting static files to distribution`);
    fsExtra.copySync(path.resolve(process.cwd(), config.staticDir), path.resolve(process.cwd(), config.distDir, './client'));
    log.debug(`executing webpack compilation`);
    await new Promise((resolve, reject) => {
      compiler.run((err, {
        stats
      }) => {
        console.log(' '); // eslint-disable-line no-console

        stats.forEach(showBuildInfo.bind(null, err));
        writePackageJsonsForAssemblies(config.distDir, nodeModulesByAssembly, peerDependencies);
        copyEnvToDist(config.distDir);
        const hasNoErrors = detectErrors(stats, false);
        void (hasNoErrors ? resolve() : reject(stats.toString('')));
      });
    });
    log.debug(`webpack compilation succeeded`);
    const {
      activeRuntimeModule,
      runtimeEntry,
      activeRuntimeOptions
    } = getEntryOptions(resolveConfig);
    log.debug(`backend entry: ${runtimeEntry}`);
    const resolveLaunchId = Math.floor(Math.random() * 1000000000);
    log.debug(`registering backend server node process`);
    const server = processRegister(['node', activeRuntimeModule, runtimeEntry, activeRuntimeOptions], {
      cwd: process.cwd(),
      maxRestarts: 0,
      kill: 5000,
      stdio: 'inherit',
      env: { ...process.env,
        RESOLVE_LAUNCH_ID: resolveLaunchId
      }
    });
    server.on('crash', reject);
    server.start();
    log.debug(`server process started with pid: ${server.pid}`);
    const {
      host,
      port
    } = JSON.parse(activeRuntimeOptions);
    log.debug(`resolve host "${host}", port "${port}"`);
    const fetchUrl = waitForUrl.bind(null, log, host, port, config.rootPath);
    let lastError;

    for (let handler of apiHandlerUrls) {
      log.debug(`request to: ${handler}`);

      try {
        await fetchUrl(handler);
        log.debug(`${handler} request completed`);
      } catch (e) {
        log.error(`${handler} ${e}`);
        lastError = e;
      }
    }

    if (typeof runAfterLaunch === 'function') {
      try {
        await runAfterLaunch(options, config);
      } catch (error) {
        lastError = error;
      }
    }

    await new Promise(resolve => server.stop(resolve));
    log.debug('Server was stopped');

    if (lastError != null) {
      throw lastError;
    }

    resolve();
  } catch (error) {
    reject(error);
  }
});

export default generateCustomMode;
//# sourceMappingURL=generate_custom_mode.js.map