"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _isomorphicFetch = _interopRequireDefault(require("isomorphic-fetch"));

var _prepare_urls = _interopRequireDefault(require("./prepare_urls"));

var _path = _interopRequireDefault(require("path"));

var _process_manager = require("./process_manager");

var _validate_config = _interopRequireDefault(require("./validate_config"));

var _get_webpack_configs = _interopRequireDefault(require("./get_webpack_configs"));

var _get_peer_dependencies = _interopRequireDefault(require("./get_peer_dependencies"));

var _webpack = _interopRequireDefault(require("webpack"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _show_build_info = _interopRequireDefault(require("./show_build_info"));

var _write_package_jsons_for_assemblies = _interopRequireDefault(require("./write_package_jsons_for_assemblies"));

var _copy_env_to_dist = _interopRequireDefault(require("./copy_env_to_dist"));

var _getLog = require("./get-log");

var _detect_errors = _interopRequireDefault(require("./detect_errors"));

var _adjustResolveConfig = _interopRequireDefault(require("./adjust-resolve-config"));

var _get_entry_options = _interopRequireDefault(require("./get_entry_options"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const waitForUrl = async (log, host, port, rootPath, apiHandlerUrl) => {
  const urls = (0, _prepare_urls.default)('http', host, port, rootPath);
  const baseUrl = urls.localUrlForBrowser;
  const url = `${baseUrl}api/${apiHandlerUrl}`;
  log.debug(`target API url to fetch ${url}`);

  while (true) {
    try {
      const response = await (0, _isomorphicFetch.default)(url);
      const result = await response.text();

      if (result !== 'ok') {
        throw [`Error communicating with reSolve HTTP server at port ${port}`, `Multiple instances of reSolve applications may be trying to run on the same port`, `${response.status}: ${response.statusText}`, `${result}`].join('\n');
      }

      break;
    } catch (e) {}

    await new Promise(resolve => setTimeout(resolve, 500));
  }
};

const generateCustomMode = (getConfig, apiHandlerUrl, runAfterLaunch) => (resolveConfig, options, adjustWebpackConfigs) => new Promise(async (resolve, reject) => {
  const apiHandlerUrls = [].concat(apiHandlerUrl);
  const log = (0, _getLog.getLog)(`start:${apiHandlerUrls.join('+')}`);

  try {
    log.debug(`validating framework config`);
    await (0, _adjustResolveConfig.default)(resolveConfig);
    const config = await getConfig(resolveConfig, options);
    (0, _validate_config.default)(config);
    log.debug(`requesting webpack configs`);
    const nodeModulesByAssembly = new Map();
    const webpackConfigs = await (0, _get_webpack_configs.default)({
      resolveConfig: config,
      nodeModulesByAssembly,
      adjustWebpackConfigs
    });
    const peerDependencies = (0, _get_peer_dependencies.default)();
    log.debug(`creating webpack compiler`);
    const compiler = (0, _webpack.default)(webpackConfigs);
    log.debug(`injecting static files to distribution`);

    _fsExtra.default.copySync(_path.default.resolve(process.cwd(), config.staticDir), _path.default.resolve(process.cwd(), config.distDir, './client'));

    log.debug(`executing webpack compilation`);
    await new Promise((resolve, reject) => {
      compiler.run((err, {
        stats
      }) => {
        console.log(' '); // eslint-disable-line no-console

        stats.forEach(_show_build_info.default.bind(null, err));
        (0, _write_package_jsons_for_assemblies.default)(config.distDir, nodeModulesByAssembly, peerDependencies);
        (0, _copy_env_to_dist.default)(config.distDir);
        const hasNoErrors = (0, _detect_errors.default)(stats, false);
        void (hasNoErrors ? resolve() : reject(stats.toString('')));
      });
    });
    log.debug(`webpack compilation succeeded`);
    const {
      activeRuntimeModule,
      runtimeEntry,
      activeRuntimeOptions
    } = (0, _get_entry_options.default)(resolveConfig);
    log.debug(`backend entry: ${runtimeEntry}`);
    const resolveLaunchId = Math.floor(Math.random() * 1000000000);
    log.debug(`registering backend server node process`);
    const server = (0, _process_manager.processRegister)(['node', activeRuntimeModule, runtimeEntry, activeRuntimeOptions], {
      cwd: process.cwd(),
      maxRestarts: 0,
      kill: 5000,
      stdio: 'inherit',
      env: { ...process.env,
        RESOLVE_LAUNCH_ID: resolveLaunchId
      }
    });
    server.on('crash', reject);
    server.start();
    log.debug(`server process started with pid: ${server.pid}`);
    const {
      host,
      port
    } = JSON.parse(activeRuntimeOptions);
    log.debug(`resolve host "${host}", port "${port}"`);
    const fetchUrl = waitForUrl.bind(null, log, host, port, config.rootPath);
    let lastError;

    for (let handler of apiHandlerUrls) {
      log.debug(`request to: ${handler}`);

      try {
        await fetchUrl(handler);
        log.debug(`${handler} request completed`);
      } catch (e) {
        log.error(`${handler} ${e}`);
        lastError = e;
      }
    }

    if (typeof runAfterLaunch === 'function') {
      try {
        await runAfterLaunch(options, config);
      } catch (error) {
        lastError = error;
      }
    }

    await new Promise(resolve => server.stop(resolve));
    log.debug('Server was stopped');

    if (lastError != null) {
      throw lastError;
    }

    resolve();
  } catch (error) {
    reject(error);
  }
});

var _default = generateCustomMode;
exports.default = _default;
//# sourceMappingURL=generate_custom_mode.js.map